// (C) Copyright 2003: Reece H. Dunn

#ifndef BOOST__IOFM__DETAIL__OUTPUT_DEDUCER__HPP
#define BOOST__IOFM__DETAIL__OUTPUT_DEDUCER__HPP
#  include <boost/io/format_objects.hpp>

   namespace boost { namespace io
   {
      namespace detail
      {
         template< int n, class T, typename FormatType >
         struct output_deducer      {};

         template< class T, typename FormatType >
         struct composite_deducer
         {
            typedef type_deducer< T >            info;
            enum{ typeID = info::type }; // workaround for BCC

            typedef output_deducer< typeID, typename info::object_type, FormatType >
                                                 type;
         };

         template< class T, typename FormatType >
         struct output_deducer< boost::io::io_type::basic, T, FormatType >
         {
            typedef type_deducer< T >            info;

            typedef basic_output                 outputter;

            static outputter generate()
            {
               outputter               o;
               return( o ); // workaround for GCC: does not like outputter()
            }
         };

         template< class T, typename FormatType >
         struct output_deducer< boost::io::io_type::pair, T, FormatType >
         {
            typedef type_deducer< T >            info;

            typedef composite_deducer< typename info::base1_type, FormatType >
                                                 base1;
            typedef composite_deducer< typename info::base2_type, FormatType >
                                                 base2;

            typedef pair_output< FormatType, typename base1::type::outputter, typename base2::type::outputter >
                                                 outputter;

            static outputter generate()
            {
               return( outputter( base1::type::generate(), base2::type::generate()));
            }
         };

         template< class T, typename FormatType >
         struct output_deducer< boost::io::io_type::nary, T, FormatType >
         {
            typedef type_deducer< T >            info;

            typedef composite_deducer< typename info::base_type, FormatType >
                                                 base;

            typedef static_nary_output< FormatType, typename base::type::outputter >
                                                 outputter;

            static outputter generate()
            {
               return( outputter( base::type::generate()));
            }
         };

         template< class T, typename FormatType >
         struct output_deducer< boost::io::io_type::container, T, FormatType >
         {
            typedef type_deducer< T >            info;

            typedef composite_deducer< typename info::base_type, FormatType >
                                                 base;

            typedef container_output< FormatType, typename base::type::outputter >
                                                 outputter;

            static outputter generate()
            {
               return( outputter( base::type::generate()));
            }
         };

         template< class T, typename FormatType >
         struct output_deducer< boost::io::io_type::array, T, FormatType >
         {
            typedef type_deducer< T >            info;

            typedef composite_deducer< typename info::base_type, FormatType >
                                                 base;

            typedef array_output< FormatType, typename base::type::outputter >
                                                 outputter;

            static outputter generate()
            {
               return( outputter( base::type::generate(), info::length ));
            }
         };
      }

      template< class T, typename FormatType = char >
      struct deducer
      {
         typedef type_deducer< T >               info;
         enum{ typeID = info::type }; // workaround for BCC

         typedef detail::output_deducer< typeID, typename info::object_type, FormatType >
                                                 type;
      };
   }}
#endif
