// (C) Copyright 2003: Reece H. Dunn 

#ifndef BOOST__IOFM__DETAIL__PAIR_TRAITS__HPP
#define BOOST__IOFM__DETAIL__PAIR_TRAITS__HPP
#  include <boost/io/detail/format_traits.hpp>

   namespace boost { namespace io { namespace detail
   {
      template< typename T >
      struct pair_traits            {};

      // formatting for characters:

      template<>
      struct pair_traits< char >
      {
         static const char             open_default;
         static const char             close_default;
         static const char             separator_default;
      };

      const char pair_traits< char >::open_default      = '(';
      const char pair_traits< char >::close_default     = ')';
      const char pair_traits< char >::separator_default = ',';

      // formatting for wide characters:

      template<>
      struct pair_traits< wchar_t >
      {
         static const wchar_t          open_default;
         static const wchar_t          close_default;
         static const wchar_t          separator_default;
      };

      const wchar_t pair_traits< wchar_t >::open_default      = L'(';
      const wchar_t pair_traits< wchar_t >::close_default     = L')';
      const wchar_t pair_traits< wchar_t >::separator_default = L',';

      // formatting for strings:

      template<>
      struct pair_traits< char * >
      {
         static char *                 open_default;
         static char *                 close_default;
         static char *                 separator_default;
      };

      char * pair_traits< char * >::open_default      = "( ";
      char * pair_traits< char * >::close_default     = " )";
      char * pair_traits< char * >::separator_default = ", ";

      template<>
      struct pair_traits< const char * >: public pair_traits< char * >
      {
      };

      // formatting for wide strings:

      template<>
      struct pair_traits< wchar_t * >
      {
         static wchar_t *              open_default;
         static wchar_t *              close_default;
         static wchar_t *              separator_default;
      };

      wchar_t * pair_traits< wchar_t * >::open_default      = L"( ";
      wchar_t * pair_traits< wchar_t * >::close_default     = L" )";
      wchar_t * pair_traits< wchar_t * >::separator_default = L", ";

      template<>
      struct pair_traits< const wchar_t * >: public pair_traits< wchar_t * >
      {
      };

      // std::basic_string format_triats

#     if !defined(BOOST_NO_TEMPLATE_PARTIAL_SPECIALIZATION)
         template< typename CharT, class Traits >
         struct pair_traits< std::basic_string< CharT, Traits > >
         {
            typedef std::basic_string< CharT, Traits >     string_type;

            static string_type         open_default;
            static string_type         close_default;
            static string_type         separator_default;
         };

         template< typename CharT, class Traits >
         std::basic_string< CharT, Traits > pair_traits< std::basic_string< CharT, Traits > >::open_default =
            std::basic_string< CharT, Traits >( pair_traits< CharT * >::open_default );

         template< typename CharT, class Traits >
         std::basic_string< CharT, Traits > pair_traits< std::basic_string< CharT, Traits > >::close_default =
            std::basic_string< CharT, Traits >( pair_traits< CharT * >::close_default );

         template< typename CharT, class Traits >
         std::basic_string< CharT, Traits > pair_traits< std::basic_string< CharT, Traits > >::separator_default =
            std::basic_string< CharT, Traits >( pair_traits< CharT * >::separator_default );
#     else
         template<>
         struct pair_traits< std::string >
         {
            static std::string         open_default;
            static std::string         close_default;
            static std::string         separator_default;
         };

         std::string pair_traits< std::string >::open_default      = std::string( "( " );
         std::string pair_traits< std::string >::close_default     = std::string( " )" );
         std::string pair_traits< std::string >::separator_default = std::string( ", " );

         template<>
         struct pair_traits< std::wstring >
         {
            static std::wstring        open_default;
            static std::wstring        close_default;
            static std::wstring        separator_default;
         };

         std::wstring pair_traits< std::wstring >::open_default      = std::wstring( L"( " );
         std::wstring pair_traits< std::wstring >::close_default     = std::wstring( L" )" );
         std::wstring pair_traits< std::wstring >::separator_default = std::wstring( L", " );
#     endif
   }}}
#endif
