// (C) Copyright 2003: Reece H. Dunn

#include <iostream>
#include <boost/io/detail/format_traits.hpp>
#include <boost/io/detail/pair_traits.hpp>

#include "print_type.hpp"              // pretty-printing of data types

template< typename FormatType >
void test_format_type()
{
   typedef boost::io::detail::format_traits< FormatType > format_type;
   typedef boost::io::detail::pair_traits<   FormatType > pairfmt_type;

   std::cout << "format_traits defaults for: " << boost::io::print_type< FormatType >() << '\n';
   std::cout << "   open      = '" << format_type::open_default      << "'" << '\n';
   std::cout << "   close     = '" << format_type::close_default     << "'" << '\n';
   std::cout << "   separator = '" << format_type::separator_default << "'" << '\n';

   std::cout << "pair_traits defaults for: " << boost::io::print_type< FormatType >() << '\n';
   std::cout << "   open      = '" << pairfmt_type::open_default      << "'" << '\n';
   std::cout << "   close     = '" << pairfmt_type::close_default     << "'" << '\n';
   std::cout << "   separator = '" << pairfmt_type::separator_default << "'" << '\n';

   std::cout << '\n';
}

#if !defined(BOOST_IOFM__NO_STD_WSTREAM)
   template< typename FormatType >
   void test_wide_format_type()
   {
      typedef boost::io::detail::format_traits< FormatType > format_type;
      typedef boost::io::detail::pair_traits<   FormatType > pairfmt_type;

      std::wcout << L"format_traits defaults for: " << boost::io::print_type< FormatType >()  << L'\n';
      std::wcout << L"   open      = '" << format_type::open_default      << L"'" << L'\n';
      std::wcout << L"   close     = '" << format_type::close_default     << L"'" << L'\n';
      std::wcout << L"   separator = '" << format_type::separator_default << L"'" << L'\n';

      std::wcout << L"pair_traits defaults for: " << boost::io::print_type< FormatType >() << L'\n';
      std::wcout << L"   open      = '" << pairfmt_type::open_default      << L"'" << L'\n';
      std::wcout << L"   close     = '" << pairfmt_type::close_default     << L"'" << L'\n';
      std::wcout << L"   separator = '" << pairfmt_type::separator_default << L"'" << L'\n';

      std::wcout << L'\n';
   }
#endif

// main test program

int main()
{
   std::cout << "testing: <boost/io/detail/format_traits.hpp>" << '\n' << '\n';

   test_format_type<       char   >();
   test_format_type<       char * >();
   test_format_type< const char * >();
   test_format_type<       std::string >();
// test_format_type< const std::string >();

#  if !defined(BOOST_IOFM__NO_STD_WSTREAM)
      test_wide_format_type<       wchar_t   >();
      test_wide_format_type<       wchar_t * >();
      test_wide_format_type< const wchar_t * >();
      test_wide_format_type<       std::wstring >();
//    test_wide_format_type< const std::wstring >();
#  endif

   return( 0 );
}
